;
;	$Id: scrt_ut71_example_LC.asm,v 1.4 2018/03/29 00:59:28 loren Exp $
;
;	A18 SCRT example for UT4/IDIOT monitors on the 1802 Membership Card.
;
;   Adapted from MPM-241, Appendix G, UT71 monitor listing.
;

R0     EQU 0
R1     EQU 1

SP     EQU 002H ;STACK POINTER
PC     EQU 003H ;PROGRAM COUNTER
CALL   EQU 004H ;CALL ROUTINE REGISTER
RETN   EQU 005H ;RETURN ROUTINE REGISTER
LINK   EQU 006H ;SUBROUTINE.DATA LINK (return stack)

;
; This block has the SCRT initialization and defines the supporting
; routines.
;
; Either include it directly in the application program or burn to an
; EPROM for convenience. 
;

	org 00100H
;
; initialization:  expects that R2 already points to the stack, (UT4 and 
;                  the UT4/IDIOT monitor will set this up, otherwise the
;                  application needs to do this beforehand).
;
;                  after init, execution will continue whereever R3
;                  pointed to on entry.
;
INITSCRT:
	DIS                  ; DISABLE INTERRUPTS
	DB    000H
	LDI   LOW  CALLR     ; SET UP SUBROUTINE CALL PC
	PLO   CALL
	LDI   HIGH CALLR
	PHI   CALL
	LDI   LOW  RETR      ; SET UP RETURN PC
	PLO   RETN
	LDI   HIGH RETR
	PHI   RETN
	SEX   SP             ; ENSURE THAT R2 IS THE STACK POINTER
	SEP   PC             ; BEGIN USER PROGRAM


;****************************************************************************
;  DESC: STANDARD SEP CALL;,A( AND RETURN
;  REG USED: SP, PC, SEP CALL;.A(,RETURN,LINK & STACK
;****************************************************************************
;
;  STANDARD CALL - USES R4 AS ITS PROGRAM COUNTER
;

EXITC:
	SEP   PC             ; GO TO IT
CALLR:
	SEX   SP             ; SET R(X)
	GHI   LINK
	STXD                 ; SAVE THE CURRENT LINK ON
	GLO   LINK
	STXD                 ;THE STACK
	GHI   PC
	PHI   LINK
	GLO   PC
	PLO   LINK
	LDA   LINK
	PHI   PC             ; PICK UP THE SUBROUTINE
	LDA   LINK
	PLO   PC             ; ADDRESS
	BR    EXITC
;
; STANDARD RETURN - USES R5 AS ITS PROGRAM COUNTER
;

EXITR:
	SEP   PC             ; RETURN TO MAIN PROGRAM
RETR:
	GHI   LINK           ; recover calling program return addr
	PHI   PC
	GLO   LINK
	PLO   PC
	SEX   SP
	INC   SP             ; SET THE STACK POINTER
	LDXA
	PLO   LINK           ; RESTORE THE CONTENTS OF
	LDX
	PHI   LINK           ; LINK
;	
; Loren: the next instruction in the original RCA listing is pretty UT71
;        specific. It's an example of how SCRT can be customized.
;	GHI   CHAR          ;  PUT THE CONTENTS OF CHAR.1 INTO D	
	                    ;  BEFORE RETURNING
	BR    EXITR
;
;
;****************************************************************************
; SCRT example, display a byte on the Membership Card LEDS. 
;
; enter with P=X=0, (monitor $P0, or CPU reset)
;****************************************************************************
 
	org 00000H
	
RETMON EQU   08000H     ; monitor entry address.

OUTVAL EQU   0FFH       ; byte to display on LEDs.

; set R3 to point at the test program's entry point.
	LDI   LOW  START
	PLO   PC
	LDI   HIGH START
	PHI   PC
	
; initialize SCRT
	LBR   INITSCRT
	
; after initialization, execution resumes here.
START:
	CALL  LEDOUT         ; test an SCRT call to the LED display routine.
	
; SCRT will return here.
; now restart the monitor with X=P=0.

	LDI   LOW  RETMON 
	PLO   R0
	LDI   HIGH RETMON
	PHI   R0
	SEX   R0
	SEP   R0             ; go now to the monitor entry point.
	
; SCRT SUBROUTINE TO OUTPUT TO MEMBERSHIP CARD LEDS, (OUT4)	
LEDOUT:
	LDI   OUTVAL         ; load the output byte.
	STR   SP             ; push to stack
	OUT   4              ; output to LEDs 
	DEC   SP             ; restore stack pointer, (OUT increments it)
	EXIT                 ; SCRT return from subroutine.

	END
;
;	scrt_ut71_example_LC.asm
;
