.. UT71.LST   https://groups.io/g/cosmacelf/files/UT71%20Project
.. ASM8 output listing for UT71.SR, errors and all David Schultz	92 kB	Feb 7 2020
.. MSGE1 error B2-> BZ corrected, LST file converted to ASM file  HRJ May 21 2020

..*************************************************************************

..NAME: UT71. VER 0.0
..DATE: 8/2/83 CHANGED HLT FROM 76 TO 60 MSEC. FIXED WRITA 10/24/83
..
..     UT71 IS A MONITOR PROGRAM DESIGNED TO EXAMINE OR ALTER MEMORY,
..     TO BEGIN PROGRAM EXECUTION AT A GIVEN LOCATION, TO PROVIDE DISK,
..     TERMINAL,AND PRINTER INTERFACE ROUTINES, AND TO PROVIDE MEANS FOR
..     COMMUNICATING FROM TERMINAL TO DISK AND SYSTEM I/O. THE MONITOR
..     COMMANDS ARE INITIATED BY TYPING D. F. I. M. S. P. T. L. B. W. R.
..     !, OR ?. THE FUNCTIONS INCLUDE MEMORY DISPLAY (D), FILL (F),
..     INSERT (I), MOVE (M), AND SUBSTITUTE (S), RUN PROGRAM (P), SELF
..     TEST (T), LOAD OPERATING SYSTEM FROM DRIVE 0 (L) OR ANY DRIVE (B),
..     WRITE (W) OR READ (R) A SECTOR, AND WRITE (!) OR READ (?) AN I/O
..     PORT. ALSO INCLUDED ARE THE STANDARD READ AND TYPE ROUTINES FOR
..     COMMUNICATION WITH THE USERS TERMINAL, AND A PARALLEL PRINTER
..     OUTPUT ROUTINE.
..     UPON STARTING UT71, THE CONTENTS OF ALL CPU REGISTERS EXCEPT R0
..     AND R1 ARE SAVED IN RAM AT #8C00.
..     UT71 RESIDES IN ROM AT 8OOO-87FF. AND MUST HAVE RAM AT 88OO-8FFF
..     (8COO-8CFF IF DISK ROUTINES NOT USED.) 
..     UT71 PROVIDES MEANS FOR FORCING KEYBOARD READS TO GO TO A COMMAND
..     FILE INTERPETER INSTEAD.
..*************************************************************************
..     SYSTEM EOUATES. CONSTANTS & OFFSETS
..*************************************************************************
..
..     COMMAND FILE INTERPETER START ADDRESS (SEE READ ROUTINE)
..
CFREAD EQU 8FFDH
..
..     REGISTER ASSIGNMENTS - GEN. & UTILITIES

SP     EQU 002H ..STACK POINTER
PC     EQU 003H ..PROGRAM COUNTER
CALL   EQU 004H ..CALL ROUTINE REGISTER
RETN   EQU 005H ..RETURN ROUTINE REGISTER
LINK   EQU 006H ..SUBROUTINE.DATA LINK
TMPRG1 EQU 007H ..TEMPORARY REGISTER
TMPRG2 EQU 00EH ..TEMPORARY REGISTER
TMPRGS EQU 009H ..TEMPORARY REGISTER
ADRPTR EQU 00BH ..HOLDS ADDR DURING STORE FROM TP
CNT    EQU 00AH ..BYTE COUNT
SRC    EQU 00BH ..SOURCE REGISTER
TPTR   EQU 00BH ..TABLE POINTER
DELAY  EQU 00CH ..DELAY ROUTINE COUNTER
PTER   EQU 00CH ..IOCB PTR
PTR    EQU 00CH ..IOCB PTR
ASL    EQU 00DH .. HEX INPUT REGISTER
DEST   EQU 00DH ..DESTINATION REGISTER
AUX    EQU 00EH ..AUX.1 HOLDS BIT TIME CONSTANT
CHAR   EQU 00FH .. STORES ASCII I/O

..     RAM / ROM ALLOCATIONS

UT71   EQU 08000H
TOPSTK EQU 008CFFH
WRAM   EQU 08C1FH

..     ASCII CONTROL CHARACTERS

NULL   EQU 000H ..NULL
COMMA  EQU 2CH ..COMMA
SEMCOL EQU 3BH ..SEMICOLON
BS     EQU 008H ..BACK SPACE
LF     EQU 00AH ..LINE FEED
CR     EQU 00DH ..CARRIAGE RETURN
EOF    EQU 013H ..END OF FILE
SPACE  EQU 020H ..SPACE
CRLF   EQU 00D0AH ..CR LF

.. CONSTANTS
BDSEL  EQU 001H ..PORT FOR TWO LEVEL I/O SELECT
LNECNT EQU 00FH ..# OF BYTES PER LINE IN DISPLAY ROUTINE
LINES  EQU 014H ..NUMBER OF LINES PER SCREEN LOAD
PGMSRT EQU 00005H ..START ADDRESS FOR INIT1
PROMPT EQU 02AH ..PROMPT CHARACTER
ROWLEN EQU 028H ..# OF CHARACTERS IN A ROW
TRMINL EQU 001H ..SELECTS THE TERMINAL
UARTBD EQU 001H ..SELECTS THE UART
URTCTL EQU 003H ..HRITE TO UART CONTROL REGISTER
CTLWRD EQU 01DH ..UART CONTROL QORD
CHARAC EQU 002H ..PORT.FOR UART QORD OUT
STATUS EQU 003H ..PORT TO READ UART STATUS

..*************************************************************************
       ORG UT71

       DIS;
       DC 0                   ..DISABLE
       LDI A.1(UT71) ; PHI R0 ..ESTABLISH PROGRAM COUNTER AT
                              ..8000 HEX

..*************************************************************************
.. REGISTER SAVE
.. SAVES CONTENTS OF THE CPU REGISTERS @#8C00
.. CLOBBERS R0 AND R1 (LEAVES 0'S AS A REMINDER)
..*************************************************************************

       A.1(WRAM)->R1.1      ..TOP OF SAVE AREA
       A.0(WRAM)->R1.0
       SEX R1
LOOP   DEC R1               ..POINT BELOW WHERE SAVED R IS TO GO
       0D0H->@-             ..LOAD SEP R0 INSTRUCTION FOR RETURN
       R1.0/2;LSDF          ..FOR EVEN VALUES OF R1
       ORI 10H              .. MAKE 9X INSTRUCTION
       ADI 81H              ..OTHERWISE 8X INSTRUCTION
       ->@R1                ..STORE FOR EXECUTION
       XOR;BNZ UT71A        ..LEAVE IF NO RAM THERE
       SEP R1               ..GO EXECUTE
       ->@-                 .. AND STORE RESULT
       R1.0-3;BNZ LOOP      ..LOOP FOR REGISTERS F - 2
       ->@-,@-,@-,@R1       ..FILL LOCATIONS FOR 0 AND 1 WITH 0
UT71A LBR INIT
..
       ORG UT71+002CH       ..PRESERVE START ADDRESS
..
START  CALL TIMALC
       AUX.1;ANI 01H        ..ECHO SET ?
       BZ SCAN1             ..BRANCH IF YES

..*************************************************************************
.. OUTPUT THE UTILITY PROMPT
..*************************************************************************


PRMPT CALL TPOFF            ..SEL GROUP 1
       CALL OSTRNG          ..TYPE SCAN MODE '*' PROMPT
       DC (CRLF)
       DC PROMPT
       DC 0

..*************************************************************************
..                     MONITOR COMMAND INTERPRETER
..      FETCHES THE ADDRESS FROM THE COMMAND TABLE AND SETS
..      THE PROGRAM COUNTER TO IT
..  REG USED: PTR. CHAR. SP, ASL
..*************************************************************************


SCNLTR SEP CALL;
       DC (READ)              ..READ COMMAND (LEAVES CHAR. IN D)

SCAN1  CHAR.1->@SP            ..GET INPUT; STORE FOR COMPARE
       A.0(TAB2-2)->TPTR.0    ..INITIALIZE TABLE POINTER
       PC.1->TPTR.1
SCAN   INC TPTR;INC TPTR      ..PT TO NEXT (FIRST) ENTRY
       LDA TPTR;BZ ERROR      ..ERROR IF END OF TABLE
       XOR                    ..LOOK FOR MATCH
       BNZ SCAN               ..LOOP IF NOT
       ->ASL.1;->ASL.0        ..ZERO CHARACTER REGISTER
       CALL TYPE6;DC ' '      ..SPACE STARTS COMMAND
       DEC SP;DEC SP          ..FAKE IT FOR THE RETURN
                              ..PICK UP COMMAND ADDRESS
       LDA TPTR;PHI LINK      ..AND TRANSFER TO THE
                              ..SUBROUTINE BY EXECUTING
       LDA TPTR;PLO LINK      ..A RETURN INSTRUCTION
       SEP R5                 ..P=3,X=2,R4 ;SEP CALL;..R5 ;RETURN,R2=#8CFF

..*************************************************************************
..                    COMMAND TABLES
..*************************************************************************

TAB2  DC 'D'
      DC (DISPLY)             ..MEMORY DISPLAY
      DC 'I'
      DC (INSERT)             ..INSERT INTO MEMORY
      DC 'M'
      DC (MOVE)               ..MOVE A BLOCK OF MEMORY
      DC 'F'
      DC (FILL)               ..FILL A BLOCK OF MEMORY
      DC 'S'
      DC (SUBST)              ..BYTE SUBSTITUTION
      DC 'P'
      DC (RUN)                ..RUN A USER PROGRAM
      DC 'T'
      DC (TEST)               ..RUN MONITOR SELF TEST
      DC 'L'
      DC (LOAD)               ..LOAD OPERATING SYSTEM FROM DRIVE 0
      DC 'B'
      DC (BOOT)               ..LOAD SAME FROM ANY DRIVE
      DC 'W'
      DC (WDISK)              ..UTILITY DISK WRITE
      DC 'R'
      DC (RDISK)              ..UTILITY DISK READ
      DC '!'
      DC (OUTPORT)            ..UTILITY OUTPUT TO PORT
      DC '?'
      DC (INPORT)             ..UTILITY INPUT FROM PORT
      DC 0
..
..*************************************************************************
..                  UTILITY ERROR MESSAGE
..           NOTE: ENTRY HERE RESETS STACK TO TOP
..REG USED: CHAR
..*************************************************************************

      ORG UT71+0085H

ERROR LDI A.0(TOPSTK);PLO SP
      LDI A.1(TOPSTK);PHI SP
      SEP CALL;
      DC (OSTRNG)
      DC (CRLF)
      DC 'ERROR'
      DC 0
PRMPT1 LBR RENTER

..*************************************************************************
..                  START OF SUBROUTINES
..*************************************************************************

..              MONITOR SUBSTITUTE FUNCTION
..       DISPLAYS THE FIRST BYE FROM THE ADDRESS GIVEN FOLLOWED
..       BY A HYPHEN. IF A HEX PAIR IS ENTERED FOLLOWED BY A SPACE,
..       IT IS SUBSTITUTED FOR THE BYTE DISPLAYED, IF A SPACE IS
..       ENTERED THERE IS NO CHANGE. IN EITHER CASE THE DATA BYTE FROM
..       THE NEXT ADDRESS WILL THEN BE DISPLAYED. THE ROUTINE IS ENDED
..       BY ENTERING A RETURN.
..REG USED:   ASL, SRC, CHAR
..*************************************************************************

SUBST  SEP CALL;
       DC (READHX)            ..INPUT ADDRESS
       GHI ASL;PHI SRC        ..SAVE START ADDRESS
       GLO ASL;PLO SRC

DECODE GHI CHAR;XRI LF        ..FIRST NON-HEX MUST BE
       BZ ADDOUT              ..A LINEFEED OR
       XRI 007H               ..TERMINATION OR
       BZ PRMPT1
       XRI 02DH               ..A SPACE
       BZ OLDDTA
       BR ERROR               ..ELSE ERROR

ADDOUT SEP CALL;
       DC (OSTRNG);
       DC CR
       DC 0
       SEP CALL;
       DC (OUT1)

OLDDTA SEP DELAY;
       DC 017H                ..WAIT TO FINISH READ
       LDN SRC;PHI CHAR       ..STAY DN SAME LINE
       SEP CALL;
       DC (TYPE2)             ..HEX OUTPUT
       SEP CALL;
       DC (OSTRNG)            ..OUTPUT A HYPHEN
       DC '-'
       DC 0
       LDN SRC;PLO ASL        ..COPY DATA FROM CELL INTO ASL

GETDTA SEP CALL;
       DC (READHX)            ..GET ANY CHANGE
       GLO ASL;STR SRC        ..RESTORE THE DATA INTO THE CELL
       INC SRC                ..OPEN THE NEXT CELL
       BR DECODE              ..EXAMINE INPUT

OUT1 SEP DELAY;
       DC 017H
       GHI SRC;PHI CHAR       ..ROUTINE TO OUTPUT A HEX PAIR
       SEP CALL;
       DC (TYPE2)             ..AND A SPACE
       GLO SRC;PHI CHAR
       SEP CALL;
       DC (TYPE2)
       SEP CALL;
       DC (OSTRNG);
       DC SPACE
       DC 0
       SEP R5
       
..*************************************************************************
..     USER CALLABLE ROUTINE TO GENERATE A DELAY. THE DELAY
..     CONSTANT IS PASSED AS AN INLINE PARAMETER. THE CALL
..     IS MADE BY DOING A SEP RC
..REG USED: DELAY, PC
..*************************************************************************

       ORG UT71+00EEH

DEXIT  SEP PC

DELAY1 LDA PC

DELAY2 SMI 1
       BZ DEXIT
       BR DELAY2

..*************************************************************************
..     SETS UP POINTER TO THE DELAY ROUTINE.
..     AND SETS UP ECHO (AUX.1'O) OR NO ECHO (AUX.1-1L
..*************************************************************************


       ORG UT71+00FEH

TIMALC GHI PC;PHI DELAY       ..DELAY SUBROUTINE ADDRESS
       LDI A.0(DELAY1)
       PLO DELAY
       SEX PC
       OUT BDSEL;
       DC UARTBD                     ..SELECT GROUP 1
       OUT URTCTL;
       DC CTLWRD                     ..SET UP UART
       SEX SP

ECHOTST 0->AUX.1                     ..DEFAULT TO ECHO (FULL DUPLEX)
       CALL OSTRNG
       DC CR,LF,'UT71 VER 0.0'       ..PRINT UT71 VERSION NUMBER


       DC CR,LF
       DC PROMPT                     ..TYPE INITIAL '*' PROMPT
       DC 0
       CALL READ                     ..WAIT FOR RESPONSE
       CHAR.1;XRI LF                 ..CHECK FOR LF
       BZ NOECHO                     ..BRANCH IF YES
       EXIT
NOECHO 1->AUX.1                      ..SET NO ECHO (HALF DUPLEX)
       EXIT


..*************************************************************************
..     READS ONE BYTE INTO CHAR.L WHEN ENTERED
..     VIA READAH, ANY HEX INPUT IS ASSEMBLED
..     INTO ASL AND DF =1, ELSE DF = 0 ON RETURN.
..REG USED: CHAR & ASL (AUX.1 HOLDS ECHO AND READ SOURCE FLAGS)
..*************************************************************************

       ORG UT71+012FH

CKDEC  ADI 7                      ..CHECK FOR ASCII DECIMAL
       BDF NFND                   ..OUT OF RANGE
       ADI 00AH                   ..SUBTRACT NET 30
       BDF FND

NFND   ADI 0                      ..SET DF = 0

REXIT  GHI CHAR                   ..PUT INPUT INTO D
       SEP R5                     .. & RETURN

READAH LDI 0;SKP                  ..SKIP TO READ1 HITH D=0

READ   GHI PC                     .. CONSTANT > 0
       PLO CHAR                   ..SAVE ENTRY POINT

       AUX.1;*2                   ..IF COMMAND FILE IS IN CONTROL;
       LBDF CFREAD                ..GO TO IT, SKIP KEYBOARD READ
       
READ1  INP STATUS;/2
       BNF READ1
       INP CHARAC                 ..READ CHARACTER
       ANI 07FH;->CHAR.1
       BZ READ1                   ..IGNORE IF ITS'S A NULL
CFRET  AUX.1;/2                   ..COMMAND FILE RETURN POINT
       BDF NEXT
RDWAIT INP STATUS;*2              ..ECHO IF ECHO BIT SET
       BNF RDWAIT
       CHAR.1->@SP
       OUT CHARAC
       DEC SP


NEXT   GLO CHAR                   ..CHECK ENTRY
       BNZ REXIT                  ..ENTERED VIA READ

CKHXE  GHI CHAR;SMI 041H          ..CHECK FOR ASCII HEX
       BNF CKDEC                  ..CHECK FOR ASCII DECIHAL
       SMI 6                      ..A THRU F
       BDF NFND                   ..NO
       ADI 010H                   ..SUBTRACT NET 37

FND    .AND.0FH->@-               ..SAVE TEMPORARILY
       GHI ASL                    ..SHIFT DATA INTO ASL
       SHL;SHL;SHL;SHL;STR SP     ..SHL 4X
       GLO ASL
       SHR;SHR;SHR;SHR
       OR;PHI ASL
       GLO ASL
       SHL;SHL;SHL;SHL
       INC SP
       OR;PLO ASL
       SMI 0                      ..SET DF = 1
       BR REXIT

..*************************************************************************
..        TYPES ONE BYTE FROM CHAR.1 AS AN ASCII
..        CHARACTER DR AS TWO HEX DIGITS. LINE FEEDS
..        ARE FOLLOWED BY SIX NULLS. USES REGISTER
..        CHAR AND A STACK LOCATION.
..        @SP-1 HOLDS OUTPUT CHARACTER.
..        CHAR.0 HOLDS THE NUMBER OF BITS (11) IN
..        ITS LOWER DIGIT AND THE FOLLOWING CODE IN
..        ITS UPPER DIGIT:
..              0 - BYTE OUTPUT _
..              1 - FIRST HEX OUTPUT
..              2 - LAST NULL OUTPUT
..              8 - LF OUTPUT
..
..REG USED: CHAR (AUX.1 HOLDS ECHO AND READ SOURCE FLAGS)
..*************************************************************************

         ORG UT71+0198H

TYPED    BR TYPE
         ORG UT71+019CH

TYPE5D   BR TYPE5

         ORG UT71+019FH

TEXIT    SEP R5

TYPE5    LDA R5                  ..PICK UP DATA
         SKP

TYPE6    LDA R6                  ..PICK UP DATA
         SKP

TYPE     GHI CHAR;->@-           ..KEEP A COPY
         XRI LF                  ..IS IT A LINE-FEED ?
         BNZ TY2
         LDI 080H                ..# BITS ADI # NULLS
         BR TY3
TYPE2    GHI CHAR;SHR;SHR;SHR;SHR
         ADI 0F6H                ..CONVERT TO HEX
         BNF TY1                 ..IF A OR >, ADD 37
         ADI 7

TY1      SMI 0C6H;->@-           ..ELSE ADD 30
         LDI 010H                ..10 ADI NO. OF BITS
         BR TY3

TY2      LDI 0                   ..NO OF BITS

TY3      PLO CHAR

BEGIN    INP STATUS;*2
         BNF BEGIN
         INC SP                  ..PT BACK TO CHARACTER
         OUT CHARAC
         DEC SP

NXCHAR   GLO CHAR;ADI 0F0H;PLO CHAR
         BNF TEXIT               ..SEP R5 IF NO MORE
         SMI 010H                ..TEST FOR ALTERNATIVES
         BZ TEXIT                ..TYPED LAST NULL
         BNF HEX1                ..TYPED FIRST HEX
         LDI 0                   ..TYPED LF OR NULL
         BR HEX3

HEX1     GHI CHAR;ANI 00FH        ..GET 2ND HEX DIGIT
         ADI 0F6H                 ..CONVERT TO HEX
         BNF HEX2                 ..IF A OR MORE.
         ADI 7                    ..ADD NET 37

HEX2     SMI 0C6H                ..ELSE ADD NET 30

HEX3     ->@-                    ..AND SAVE
         BR BEGIN

..*************************************************************************
..                  INPUT OPTION
..       ALLOWS ENTRY OF EITHER STARTING AND ENDING
..       ADDRESSES OR BYTE COUNT. SEP R5S WITH THE
..       STARTING ADDRESS IN REG SRC AND THE BYTE
..       COUNT IN REG CNT. RETURNS HITH DF =1
..       IF SYNTAX ERROR EXISTS
..REG USED: ASL. SRC. CHAR. CNT
..*************************************************************************


          ORG UT71+0200H

OPTION    SEP CALL;
          DC (READHX)             ..GET THE STARTING ADDRESS
          GHI ASL;PHI SRC         ..AND SAVE IT
          GLO ASL;PLO SRC
          LDI 0;PLO ASL;PHI ASL   ..CLEAR THE INPUT REG
          GHI CHAR;XRI SPACE      ..FIRST NONSMI HEX MUST
          BZ CNTIN                ..BE A SPACE OR
          XRI 00DH                ..A HYPHEN
          BNZ PRMPT2              ..ELSE SYNTAX ERROR
          SEP CALL;
          DC (READHX)             ..EXPECT ENDING ADDRESS

BYTCNT    GLO SRC;STR SP          ..CALCULATE THE BYTE COUNT
          GLO ASL;SM;PLO CNT
          GHI SRC;STR SP
          GHI ASL;SMB;PHI CNT
          BDF EXITOK              ..CHECK FOR SRC < ASL

INVERT    GLO ASL;STR SP          ..ELSE EXCHANGE THE CONTENTS OF
          GLO SRC;PLO ASL         ..SRC AND ASL
          LDN SP;PLO SRC
          GHI ASL;STR SP
          GHI SRC;PHI ASL
          LDN SP;PHI SRC
          BR BYTCNT               ..RECALCULATE


CNTIN     SEP CALL;
          DC (READHX)             ..INPUT THE BYTE COUNT
          GLO ASL;SMI 1;PLO CNT
          GHI ASL;SMBI 0;PHI CNT
          BDF EXITOK
          INC CNT
EXITOK    SEP R5                  ..RETURN WHEN DONE

..*************************************************************************
..                    FILL ROUTINE
..        LOADS MEMORY BEGINNING AT ADDRESS CONTAINED
..        IN SRC NITH DATA CONTAINED IN ASL 0 FOR
..        THE NUMBER OF BYTES SPECIFIED BY CNT.
..        USER CALLABLE Q USRFIL.
..REG USED: ASL, SRC, CNT ,CHAR
..*************************************************************************

FILL     SEP CALL;
         DC (READAD)                  ..GET THE ADDRESSES
         SEP CALL;
         DC (USRFIL)                  ..CALL THE MOVE

PRMPT2   LBR RENTER                   ..GOTO UT71 AND PROMPT

NXTCEL   INC SRC                      ..POINT TO NEXT CELL
         DEC CNT ..REDUCE BYTE COUNT   ..LOAD THE DATA;USER ENTRY PT.
USRFIL   GLO ASL;STR SRC              ..LOOP UNTIL COUNT = 0
         GLO CNT
         BNZ NXTCEL
         GHI CNT
         BNZ NXTCEL
         SEP R5                       ..EXIT THE CALL

..*************************************************************************
..                  MOVE ROUTINE
..       COPIES A BLOCK OF MEMORY FORM ONE CONTINUOUS AREA
..       TO ANOTHER CONTINUOUS AREA IN MEMORY. THERE IS NO
..       RESTRICTION AS TO THE DIRECTION OF THE MOVE AND THE
..       AREAS MAY OVERLAP.
..REG USED: SRC, DEST, CHAR, & CNT
..*************************************************************************

USRMOV   SEX SP
         GLO SRC;STR SP               ..TEST THE RELATIVE POSITION
         GLO DEST;SM                  ..OF SOURCE & DESTINATION
         BNZ DIRECT                   ..NOT EQUAL!       
         GHI SRC;STR SP               ..RETURN IF THEY ARE EQUAL
         GHI DEST;SMB
         BZ USRBYE                    ..EXIT TO CALLER

DIRECT   GLO SRC;STR SP               ..ELSE TEST FOR UP OR DOWN
         GLO DEST;SM                  ..DIRECTION OF THE MOVE
         GHI SRC;STR SP
         GHI DEST;SMB
         BDF MOVUP
MOVDN    LDN SRC;STR DEST             ..DO THE MOVE DOWN AND
         GLO CNT                      ..AND CHECK IF DONE
         BNZ MOVDN1
         GHI CNT
         BZ USRBYE                    ..EXIT TO CALLER

MOVDN1   INC SRC;INC DEST             ..ADJUST THE POINTERS
         DEC CNT                      ..REDUCE THE BYTE COUNT             
         BR MOVDN                     ..FINISHED

MOVUP    GLO CNT;STR SP               ..SET THE POINTERS TO THE
         GLO SRC;ADD;PLO SRC          ..TOP OF MOVE AREAS
         GHI CNT;STR SP
         GHI SRC;ADC;PHI SRC
         GLO CNT;STR SP
         GLO DEST;ADD;PLO DEST
         GHI CNT;STR SP
         GHI DEST;ADC;PHI DEST
         BNF UP

ERRGO    BR USRBYE+1                  ..EXIT DF=1 IF OVERFLON

UP       LDN SRC;STR DEST             ..DO THE MOVE UP
         GLO CNT                      ..AND CHECK IF DONE
         BNZ UP1
         GHI CNT
         BZ USRBYE                    ..EXIT TO CALLER

UP1      DEC SRC;DEC DEST;DEC CNT     ..ADJUST THE POINTERS
         BR UP

USRBYE   SHR                          ..SET DF=0 IF A<FFFF
         SEP R5                       ..EXIT TO CALLER      

..*************************************************************************
..       STARTS A USER PROGRAM WITH SPECIFIED ADDRESS
..       IN REGISTER 0 AND X=O.
..REG USED: CHAR, ASL, R0
..*************************************************************************

RUN     SEP CALL;
        DC (READHX)             ..LOOK FOR STARTING ADDRESS
        XRI CR                  ..FIRST NON-HEX MUST BE A
        LBNZ ERROR              ..CR, ELSE SYNTAX ERROR

RUN1   GHI ASL;PHI R0           ..GET THE ADDRESS
       GLO ASL;PLO R0
       SEX R0
       SEP R0                   ..AND GO!

..*************************************************************************
..                       GENERAL REENTER ROUTINE
..*************************************************************************

RENTER LDI A.0(RENTR1);PLO R0    ..CAN BE ENTERED NITH X AND P
       LDI A.1(RENTR1);PHI R0    ..SET TO ANYTHING AND RESETS
       SEP R0                    ..ALL THE SCRT REGISTERS
RENTR1 LDI A.0(PRMPT);PLO PC
       LDI A.1(PRMPT);PHI PC
       LBR ENTER2

..*************************************************************************
..                             OUTPUT
..     FORMATS AND OUTPUTS MEMORY DATA BEGINNING
..     AT THE ADDRESS IN REG SRC FOR THE NUMBER
..     OF BYTES SPECIFIED IN REG CNT
..REG USED: SRC, CNT, CHAR
..*************************************************************************

DISPLY SEP CALL;
       DC (OPTION)               ..GET STARTING ADDRESS
       XRI CR                    ..TERMINATE WITH CR
       LBNZ ERROR

OUTPUT SEP CALL;
       DC (OSTRNG);
       DC LF
       DC 0                      ..START ON A NEW LINE
       SEP CALL;
       DC (OUT1)                 ..OUTPUT THE ADDRESS OF THE
..                                 CURRENTLY OPENED CELL

SPCOUT SEP CALL;
       DC (OSTRNG)
       DC SPACE
       DC 0

DATOUT LDA SRC;PHI CHAR          ..RETRIEVE THE CELL DATA
       SEP CALL;
       DC (TYPE2)                ..AND OUTPUT IT
       GLO CNT                   ..DETERMINE IF THE
       BNZ NOTDON ..REGUESTED NO. OF BYTES
       GHI CNT ..HAVE BEEN SENT
       BZ PRMPT2 ..GET A NEW COMMAND

NOTDON DEC CNT ..DEC THE BYTE COUNT
       GLO SRC;ANI LNECNT
       BNZ SAMELN ..END OF CURRENT LINE?
       SEP CALL;
       DC (OSTRNG)
       DC ';'
       DC CR
       DC 0
       BR OUTPUT

SAMELN SHR
       BDF DATOUT                .. WITHIN PAIR
       BR SPCOUT ..ELSE BETWEEN PAIRS

..*************************************************************************
..     FILLS ASL AS LONG AS HEX DIGITS ARE ENTERED
..*************************************************************************

READHX SEP CALL;
       DC (READAH)
       BDF READHX
       SEP R5

..*************************************************************************
..     MOVE COMMAND
..     CALLS USRMOV AND REQUESTS SRC&DEST ADDR'S
..*************************************************************************

       ORG UT71+02F7H

MOVE   SEP CALL;
       DC (READAD)                  ..GET SRC&DEST ADDR'S
       SEP CALL;
       DC (USRMOV)    ..DO THE MOVE
       LBDF ERROR                   ..ERROR IF OVER FFFF ON MOVE
       LBR PRMPT2                   ..IF OK,GOTO UT71 PROMPT

..*************************************************************************
..     SUBROUTINE TO GET THE ADDRESSES FOR OTHER ROUTINES
..*************************************************************************

READAD SEP CALL;
       DC (OPTION)                  ..DETERMINE THE MODE
       XRI SPACE                    ..MUST BE A SPACE
       BNZ ERR1                     ..ELSE ERROR
       PLO ASL;PHI ASL              ..CLEAR INPUT REGISTER
       SEP CALL;
       DC (READHX)                  ..INPUT THE CONSTANT
       XRI CR                       ..'CR' TERMINATES
       BNZ ERR1                     ..ELSE ERROR
       SEP R5

..*************************************************************************
..     FILLS ASL UNTIL A CARRIAGE RETURN IS ENTERED
..*************************************************************************

READCR SEP CALL;
       DC (READAH)
       XRI CR
       BNZ READCR
       SEP R5

..*************************************************************************
..      OSTRNG
..*************************************************************************

MSGE   LDI 0EFH;PLO DELAY
       LDI 080H;PHI DELAY

MSGE1  LDA LINK;PHI CHAR
       BZ EXITM  ..B2 error corrected HRJ
       SEP CALL;
       DC (TYPED)
       BR MSGE1

EXITM  SEP R5

..*************************************************************************
..    THIS ROUTINE PRINTS TO THE LINE PRINTER THE CONTENTS OF RF.L
..    -IT SUPRESSES PRINTING OF THE LINE FEEDS. AND REPLACES CARRIAGE
..    RETURNS WITH A CR-LF PAIR. NORMALLY. THIS ROUTINE RETURNS HITH
..    THE DFLAG SET. BUT IF THE CHARACTER IN RF.1 HAS A 0C3 (END OF
..    FILE). THE DFLAG HILL BE RESET ON RETURN
..*************************************************************************

PRNTRF  GHI RF;XRI 0AH             ..IF LINE FEED. EXIT
        BZ EXITDF
        GHI RF;XRI 13H             ..IF DC3, EXIT
        BZ EXITEF
PRINT1  GHI RF;XRI 0FFH;STR R2     ..INVERT DATA
        B1 $                       ..WAIT UNTIL READY
        OUT 6;DEC R2               ..OUTPUT CHARACTER
        GHI RF;XRI 0DH             ..CARRIAGE RETURN ?
        BNZ EXITDF                 ..ND, EXIT
        LDI 0AH;PHI RF             ..YES. PRINT A LINE FEED
        BR PRINT1
EXITDF  LDI 1;SHR                   ..SET DFLAG
        EXIT
EXITEF  SHR                         ..RESET DFLAG
        EXIT

..*************************************************************************
       ORG UT71+035DH

PRMPT5 LBR RENTER
ERR1   LBR ERROR                   ..GENERAL FOR THIS PAGE

..*************************************************************************
..DESC: STANDARD SEP CALL;,A( AND RETURN
..REG USED: SP,PC,SEP CALL;,A(,RETURN,LINK & STACK
..*************************************************************************

        ORG UT71+0363H

..                    STANDARD CALL

EXITC  SEP PC                       ..GO TO IT

CALLR  SEX SP     ..SET R(X)
       GHI LINK;STXD                ..SAVE THE CURRENT LINK ON
       GLO LINK;STXD                ..THE STACK
       GHI PC;PHI LINK
       GLO PC;PLO LINK
       LDA LINK;PHI PC              ..PICK UP THE SUBROUTINE
       LDA LINK;PLO PC              ..ADDRESS
       BR EXITC

..                  STANDARD RETURN

EXITR  SEP PC                       ..RETURN TO MAIN PGM

RETR   GHI LINK;PHI PC
       GLO LINK;PLO PC
       SEX SP;INC SP                ..SET THE STACK POINTER
       LDXA;PLO LINK                ..RESTORE THE CONTENTS OF
       LDX;PHI LINK                 ..LINK
       GHI CHAR                     ..PUT THE CONTENTS OF CHAR.1 INTO D
                                    ..BEFORE RETURNING
       BR EXITR

..*************************************************************************
..                REGISTER INITIALIZATION ROUTINE
..     INITIALIZES REGISTER C TO THE DELAY ROUTINE; REG 2 AS A STACK
..     POINTER TO LOCATION 3CFF HEX. REG 4 TO CALL. REG 5 TO RETURN
..     AND REG 3 AS PROGRAM COUNTER. FOR ENTER1 REG 3 IS 0005: FOR
..     ENTER2 REG 3 MUST BE PRESET.
..REG USED: PC, DELAY, CALL, RETURN, SP
..*************************************************************************

INIT   LDI A.0(START);PLO PC
       LDI A.1(START);PHI PC
       BR ENTER2

ENTER1 LDI A.0(PGMSRT);PLO PC
       LDI A.1(PGMSRT);PHI PC

ENTER2 LDI A.0(DELAY1);PLO DELAY       ..DELAY ROUTINE
       LDI A.1(DELAY1);PHI DELAY
       LDI A.1(CALLR);PHI CALL;PHI RETN
       LDI A.0(CALLR);PLO CALL
       LDI A.0(RETR);PLO RETN
       LDI A.0(TOPSTK);PLO SP
       LDI A.1(TOPSTK);PHI SP
       SEX SP;SEP PC

..*************************************************************************
..                 HEX BYTE INSERT ROUTINE
..     INSERTS HEX PAIRS INTO MEMORY STARTING AT A SPECIFIED
..     ADDRESS. AFTER A "5" ALL IS IGNORED UNTIL A RETURN
..     THEN A NEW ADDRESS IS EXPECTED. ANY NON-HEX DATA IS
..     IGNORED BETWEEN HEX PAIRS BUT NOTHING IS PERMITTED
..     BETWEEN MEMBERS OF THE PAIR. ROUTINE IS TERMINATED
..     HITH A RETURN. EXCEPT AFTER A                                                                                                                               5".
..REG USED: ASL, SRC, CHAR
..*************************************************************************

INSERT  0->ASL.1,ASL.0                 ..CLEAR INPUT REGISTER
INSERT1 CALL READAH
        BNF INSERT1                    ..IGNORE INPUTS UNTIL FIRST HEX
        CALL READHX                   ..THEN INPUT UNTIL FIRST NON-HEX
        .XOR.' ';BNZ ERR1              ..IT MUST BE A SPACE
        ASL.1->SRC.1
        ASL.0->SRC.0                   ..INPUTS UERE THE STARTING ADDRESS

NXTCHR  CALL READAH
        BNF NTDATA                     ..IF NEXT INPUT IS HEX
        CALL READAH                    ..GET A SECOND
        BNF ERR1                       ..WHICH MUST ALSO BE HEX
        ASL.0->@SRC;INC SRC            ..AND STORE HEX PAIR INTO MEMORY
        BR NXTCHR                      ..LOOK FOR MORE

NTDATA  .XOR.CR;BZ PRMPT5             ..IF INPUT HAS CR5 LEAVE
        .XOR.(';'.XOR.CR)
        BNZ COMCHK                     ..IF INPUT HAS ';',
        CALL READCR                    ..IGNORE EVERYTHING UNTIL CR
        CALL TYPE6;DC LF               ..ADD LINEFEED
        BR INSERT                      ..START NEXT LINE NITH NEH ADDRESS

COMCHK  .XOR.(','.XOR.';')
        BNZ NXTCHR                     ..IF INPUT HAS ',',
        CALL READCR                    ..IGNORE EVERYTHING UNTIL CR
        CALL TYPE6;DC LF               ..ADD LINEFEED
        BR NXTCHR                      ..START NEW LINE WITHOUT NEW ADDRESS

..*************************************************************************
..                        SELECT GROUP 1
..*************************************************************************

TPOFF   SEX PC
        OUT BDSEL
        DC TRMINL
        SEP R5

..*************************************************************************
..                      UTILITY ENTRY TABLE
..*************************************************************************

        ORG UT71+03F0H

OSTRNG  LBR MSGE
INIT1   LBR ENTER1
INIT2   LBR ENTER2
GOUT71  LBR RENTER
CKHEX   LBR CKHXE


..*************************************************************************
..                      DISK I/O ROUTINES
..*************************************************************************

..REGISTER EQUATES

DMAPTR  EQU 0                    ..DMA POINTER
INTPC   EQU 1                    ..INTERRUPT PC
IOCBPTR EQU 7                    ..IOCB POINTER
CMDCNT  EQU 8                    ..NO. OF COMMAND WORDS
TRKCNT  EQU 9                    ..TRACK COUNT DURING LOAD
PARA    EQU 0FH                  ..PARAMETER BLOCK POINTER

.. RAM EQUATES

RAMADR  EQU UT71+800H            ..BEGINNING OF RAM
IOCB    EQU 8F00H                ..LOADER IOCB
STA0    EQU 8F10H                ..STATUS 0

..MICRODOS EQUATES

STK     EQU 0BFFFH               ..TOP OF STACK
CAL     EQU 091BCH               ..CALL ADDRESS
RET     EQU 0919CH               ..RETURN ADDRESS
ECHOTP  EQU 0910AH               ..ECHO STATUS
ENTRY   EQU 09040H               ..ENTRY ADDRESS INTO MICRODOS
MICRO   EQU 0A843H               ..'MICRODOS' NAME IN DP SYS

..DISK BOARD I/O EQUATES

DISKSEL EQU 1                    ..BOARD SELECTION
DMASEL  EQU 4                    ..DMA DIRECTION SELECT
TERMCNT EQU 4                    ..TERMINAL COUNT/ABORT
NECSTA  EQU 4                    ..MAIN STATUS REGISTER
COMMAND EQU 5                    ..COMMAND/DATA REGISTER
DATA    EQU 5                    ..COMMAND/DATA REGISTER
BYTECNT EQU 7                    ..BYTE COUNT SELECT

..NEC COMMANDS

SPCMD   EQU 03H                  ..SPECIFY
RCCMD   EQU 07H                  ..RECALIBRATE
SKCMD   EQU 0FH  ..SEEK
RDCMD   EQU 46H  ..READ
WTCMD   EQU 45H  ..WRITE
SISCMD  EQU 08H  ..SENSE INTERRUPT STATUS
INVCMD  EQU 00H  ..INVALID COMMAND

..DISK DATA CONSTANTS

BC      EQU 04H                  ..BYTE COUNT
N       EQU 02H  ..N
EOT     EQU 09H  ..EOT
GPL3    EQU 1BH  ..GPLB
DTL     EQU 0FFH  ..DTL
FM      EQU 40H  ..DENSITY
SRT     EQU 10H  ..STEP RATE: 15 MS
HLT     EQU 3CH  ..HEAD LOAD TIME1 60 MS
HUT     EQU 0FH  ..HEAD UNLOAD TIME: 240 MS
DMA     EQU 00H  ..DMA OPERATION
DMANOP  EQU 00H  ..NO DMA OPERATION
CRCREAD EQU 01H  ..CRC READ
DMAO    EQU 02H  ..DISK WRITE
DMAI    EQU 03H  ..DISK READ-
RCA     EQU 01H  ..GROUP SELECT 1
NEC     EQU 08H  ..GROUP SELECT 8
MAXTRK  EQU 70  ..NUMBER OF TRACKS ON SONY
MAXSEC  EQU 09  ..9 SECTORS / TRACK

..*************************************************************************
..
..                      LOAD MICRODOS
..
        ORG UT71+0400H
..
BOOT    CALL READAH;BNF BOOT       ..GET DRIVE # (WAIT UNTIL HEX KEY)
LOAD    SP.0->PARA.0;SP.1->PARA.1  ..ENTER FROM                                                                                                                               L                                                                                                                              . ASSUME DRIVE = O
        ASL.0->@-                  ..DRIVE # IS @ PARA
        .AND.0FCH;LBNZ ERROR       ..ERROR IF DRIVE > 3
        ->DMAPTR.0;90H->DMAPTR.1   ..BEGINNING OF MICRODOS AREA
        A.0(TKTABL)->TRKCNT.0
        A.1(TKTABL)->TRKCNT.1      ..TABLE FOR MICRODOS LOAD
        CALL SPECIFY               ..SET UP DRIVE PARAMETERS
        BR LOAD1
..*************************************************************************

        ORG UT71+421H

        LBR USRMOV                 ..FOR COMPATIBILITY WITH UT62
                                   .. MOVE COMMAND

..*************************************************************************

TKTABL  DC 1,2,28                  ..TABLE CONTAINS TRACK #. STARTING
        DC 2,0,36                  ..SECTOR # (-1). AND BYTE COUNT (X128)
        DC 3,0,32                  ..FOR ALL MICRODOS LOAD
        DC 0                       ..0 ENDS TABLE
LOAD1   CALL SEEKA                 ..FIRST SEEK IS RECAL (ASL.1 HAS 0)
        ASL.0;BNZ NOLOAD           ..EXIT IF ABNORMAL TERM
        @TRKCNT!
LOAD2   ->ASL.1                    ..SETUP TRACK #
        @TRKCNT!->ASL.0            ..AND SECTOR 0
        CALL SEEKA                 ..SEEK ALSO SETS UP FOR READ
        ASL.0;BNZ NOLOAD           ..EXIT IF ABNORMAL TERM
        A.0(IOCB+1)->IOCBPTR.0
        A.1(IOCB+1)->IOCBPTR.1     ..POINT TO BYTE COUNT IN IOCB
        @TRKCNT!->@IOCBPTR         ..LOAD IT FROM TABLE
        CALL READA                 ..GO READ
        ASL.0;BNZ NOLOAD           ..EXIT IF ABNORMAL TERM
        @TRKCNT!;BNZ LOAD2         ..REPEAT FOR ALL TRACKS

..*************************************************************************


MICTST  A.0(MICRO)->IOCBPTR.0      ..POINT AT 'MICRODOS' IN OP SYS
        A.1(MICRO)->IOCBPTR.1
        SEX IOCBPTR
        @IOCBPTR!           ..GET 'M' AND POINT X AT 'I'
        XOR     ..EXCLUSIVE OR 'M' AND 'I'
        INC IOCBPTR    ..POINT AT 'C'
        XOR    ..EXCLUSIVE OR 'M/I' AND 'C'
        XRI 47H    ..HAS 'MIC' THERE ?
        BNZ NOLOAD    ..OP SYS IS NOT IN MEMORY

..*************************************************************************


LOADOK  CALL OSTRNG
        DC 0D0AH,0AH,00
        A.0(STK)->SP.0             ..SET UP OP SYS STACK POINTER,
        A.1(STK)->SP.1    ..CALL, RETURN, AND ECHO STATUS
-  ***
        A.0(CAL)->CALL.0
        A.0(RET)->RETN.0
        A.0(ECHOTP)->IOCBPTR.0
        A.1(CAL)->CALL.1
        ->RETN.1 
        ->IOCBPTR.1
        AUX.1->@IOCBPTR
        LBR ENTRY                  ..GO TO OPERATING SYSTEM


..*************************************************************************

NOLOAD  CALL OSTRNG                ..PRINT NOT LOADED MESSAGE
        DC LF,'MICRODOS NOT LOADED',0



        LBR RENTER                 ..GO BACK TO MONITOR (FIXES SP)

..*************************************************************************
..
.. SPECIFY SETS UP DRIVE PARAMETERS
..
SPECIFY  A.0(IOCB+4)->IOCBPTR.0    ..RESET POINTER
         A.1(IOCB+4)->IOCBPTR.1
         SEX IOCBPTR
         LDI (HLT.OR.DMA);->@-     ..HEAD LOAD TIME AND DMA
         LDI (SRT.OR.HUT);->@-     ..STEP RATE AND HEAD UNLOAD TIME
         SPCMD->@-                 ..SPECIFY COMMAND
         0FFH->@-                  ..CLEAR BYTE COUNT
         DMANOP->@-                ..DMANOP
         3->CMDCNT.0               ..BYTES IN COMMAND SEQUENCE
         CALL CMD
         EXIT
..*************************************************************************
..
.. RECAL RECALIBRATES DRIVE: ENTER HITH DRIVE # IN DMAPTR.0
.. WAIT MUST BE USED AFTER RECAL
..
RECAL    A.0(IOCB+3)->IOCBPTR.0    ..POINT AT UNIT IN IOCB
         A.1(IOCB+3)->IOCBPTR.1
         SEX IOCBPTR
         DMAPTR.0->@-              ..HEAD AND UNIT
         RCCMD->@-                 ..RECALIBRATE COMMAND
         DEC IOCBPTR
         DMANOP->@-                ..DMANOP
         2->CMDCNT.0
         CALL CMD
         EXIT
..*************************************************************************
..       COMMAND ROUTINE OUTPUTS COMMAND WORDS FROM @ IOCB FOR NUMBER
..       SPECIFIED IN CMDCNT.0. IT CLEARS SERVICE REOUEST FROM 765 FIRST.
.. 	 AND CLEARS OUT ANY RESULTS THAT ARE PENDING
..
CMD      SEX PC
         OUT DISKSEL;DC NEC             ..SELECT DISK GROUP
         BN3 CMD1 ..IF 765 WANTS SERVICE
CMD2     INP NECSTA;*2;BNF CMD2 ..WAIT FOR RQM
         OUT DATA;DC SISCMD ..DO SENSE INTERRUPT COMMAND
CMD4     NOP;NOP     ..WAIT FOR 765
CMD1     INP NECSTA;*2;BNF CMD1 ..WAIT FOR RQM
         *2;BNF CMD3    ..IF 765 WANTS TO OUTPUT STATUS.
         INP DATA;BR CMD4 ..GET STATUS BYTES UNTIL CLEAR
CMD3 B3 CMD2     ..LOOP IF SERVICE STILL WANTED
         A.0(IOCB)->IOCBPTR.0 ..POINT AT IOCB
         A.1(IOCB)->IOCBPTR.1
         SEX IOCBPTR
         OUT DMASEL                     ..DIRECTION OF DMA
         OUT BYTECNT ..LATCH BYTE COUNT
CMD6     SEX SP 
CMD5     INP NECSTA;*2;BNF CMD5..WAIT FOR RQM
         SEX IOCBPTR
         OUT COMMAND..OUTPUT COMMAND WORDS
         DEC CMDCNT ..UNTIL COUNTER = 0
         CMDCNT.0; BNZ CMD6
         SEX PC
         OUT DISKSEL;DC RCA..SELECT GROUP 1
         EXIT
..*************************************************************************
..
..       FOR COMPATABILITY WITH UT21 LINE PRINTER ROUTINE
..
         ORG UT71+050EH
..
         LBR PRNTRF
..*************************************************************************
..       WAIT ROUTINE WAITS FOR SERVICE REOUEST FROM 765. TIMES OUT IF NONE
.. 	 IF RESULT OF READ/WRITE. INPUTS STATUS BYTES. IF RESULT OF SEEK OR
.. 	 RECAL. DOES SENSE INTERRUPT STATUS FIRST. IF WRONG DRIVE. REPEATS
.. 	 USES CRC DMA CYCLE TO CLEAR DMA REOUEST IN CASE OF SERIOUS OVER-RUN.

WAIT     0FFH->IOCBPTR.1                ..SET UP TIMER
WAIT2    SEX PC
         OUT DISKSEL;DC NEC
         B3 WAIT1..SERVICE DISK IF EF3
         DEC IOCBPTR;IOCBPTR.1 ..IF NO REOUEST IN 1.3 SE05
         BNZ WAIT2 ..(2.5 SEC IF 2.5 MHZ CLK)
WAIT1    A.0(STA0)->IOCBPTR.0 ..SET UP RESULT POINTER
         A.1(STA0)->IOCBPTR.1
         80H->@IOCBPTR..EXIT WITH TERM ERROR
         BN3 ENDWAIT

WAIT5    SEX IOCBPTR..START RESULT SERVICE
WAIT3    INP NECSTA;*2;BNF WAIT3 ..WAIT FOR ROM
         ANI 20H;BNZ WAIT4 ..IF BUSY BIT LOW (END OF SEEK)
         SEX PC
         OUT COMMAND;DC SISCMD..DO SENSE INTERRUPT COMMAND
         NOP;BR WAIT5 ..AND SERVICE THAT RESULT

WAIT4    INP DATA;->@SP ..FIRST STATUS IS STO (SAVE)
WAIT8    IRX;NOP
WAIT6    INP NECSTA;*2;BNF WAIT6..WAIT FOR ROM
         *2;BNF WAIT7    ..IF DIRECTION BIT STILL = IN.
         INP DATA;BR WAIT8 ..INPUT STATUS AND LOOK AGAIN

WAIT7    A.0(IOCB+3)->IOCBPTR.0
         A.1(IOCB+3)->IOCBPTR.1..WHEN ALL RESULTS INPUT. CHECK
         @SP.XOR.@.AND.3 ..IF DRIVE NUMBER FROM STO MATCHES
         BNZ WAIT ..THAT FROM COMMAND. REPEAT IF NO
         SEX PC
         @SP.AND.0C0H..IF COMMAND TERMINATION (FROM ST0)
         BZ ENDWAIT ..WASN'T NORMAL
         OUT COMMAND;DC INVCMD ..DO INVALID COMMAND
         NOP;NOP ..(WANTS ONE RESULT)
WAIT9    INP NECSTA;*2;BNF WAIT9 ..WAIT FOR RQM
         OUT DMASEL;DC CRCREAD ..ENABLE PHONY DMA IN CIRCUIT
         OUT DMASEL;DC DMANOP ..(INPUTS RESULT IF DRQ HUNG HIGH
         NOP;NOP       ..FROM DMA OVER-RUN)
WAIT10   INP NECSTA;*2;BNF WAIT10 ..WAIT FOR RQM
         *2;BNF ENDWAIT    ..IF DRQ DIDN'T INPUT RESULT
         INP DATA ..DO IT ANYNAY
ENDWAIT  OUT DISKSEL;DC RCA ..BACK TO TERMINAL GROUP
         EXIT ..EXIT DISK SERVICE ROUTINE

..*************************************************************************

..                         SEEK ROUTINE

..WHEN ENTERED AT SEEKST, CALCULATES TRACK, SECTOR NO. FROM @ PARA +1 AND +2.
..CAN ALSO BE ENTERED AT SEEKA WITH ASL.1 = TRACK NO., ASL.0 = SECTOR NO.-1
..(STILL USES @PARA FOR UNIT #)
..DOES RECAL IF TRACK = 0. SETS UP IOCB FOR LATER READS OR HRITES
..
..       PARAMETER BLOCK POINTER: @ PARA = UNIT NO.
..         	                  @ +1 = PSN HIGH BYTE, @ +2 = LOW BYTE
.. 				  @ +3 = BUFFER ADDRESS H.B.. @ +4 = L.B
..       SEEKST USES IOCBPTR.0 FOR COUNTER,.1 FOR TEMP STORE.
.. 	 AUX.0 AS DIVIDEND H.B., ASL.1 FOR TRACK #, ASL.0 FOR SECTOR # (-1)
.. 	 ASL.0 HOLDS TERMINATION RESULT AT END
..
SEEKST   0->ASL.1                   ..CLEAR FUTURE RESULT
         INC PARA;@PARA!->AUX.0     ..PSN HIGH BYTE
         @PARA->ASL.0     ..AND LOW BYTE
-  ***
         DEC PARA;DEC PARA
         2->@-;40H->@SP    ..DIVISOR = 9 SHIFTED LEFT 6 TIMES
         7->IOCBPTR.0     ..SUBTRACT AND SHIFT 7 TIMES
-  ***
SUBLP    ASL.0-@->IOCBPTR.1
         INC SP;AUX.0-"@    ..DIVIDEND - DIVISOR
         BM SHRES     ..IF NOT -
         ->AUX.0;IOCBPTR.1->ASL.0   ..STORE NEH DIVIDEND
SHRES    ASL.1*2"->ASL.1     ..SHIFT NO BORROU INTO RESULT
-  ***
         @/2->@-;@/2"->@SP     ..SHIFT DIVISOR RIGHT
-  ***
         DEC IOCBPTR
         IOCBPTR.0;BNZ SUBLP        ..LOOP 7 TIMES
         INC SP            ..FIX STACK POINTER
..
SEEKA   A.0(IOCB+10)->IOCBPTR.0     ..POINT AT IOCB DTL VALUE
        A.1(IOCB+10)->IOCBPTR.1
        SEX IOCBPTR
        DTL->@-                     ..DTL
        GPL3->@-     ..GPLB
-  ***
        EOT->@-     ..EOT
-  ***
        N->@-     ..N
-  ***
        ASL.0+1->@-     ..SECTOR + 1
-  ***
        00H->@-     ..HEAD 0
-  ***
        ASL.1->@-     ..TRACK
-  ***
        @PARA->@-     ..HEAD AND UNIT
-  ***
        3->CMDCNT.0     ..3 COMMAND BYTES IN SEEK
-  ***
        ASL.1;BNZ SEEK5     ..IF TRACK IS 0.
        DEC CMDCNT     ..ONLY 2 COMMAND BYTES
      RCCMD     ..AND RECAL COMMAND INSTEAD
-  ***
        LSKP
SEEK5    SKCMD->@-            ..OF SEEK COMMAND
        BC->@-     ..BYTE COUNT
-  ***
        DMANOP->@IOCBPTR     ..DMANOP
-  ***
        CALL CMD
        CALL WAIT
        A.1(STA0)->IOCBPTR.1        ..POINT AT RESULT STATUS
        A.0(STA0)->IOCBPTR.0
        @IOCBPTR.AND.0C0H           ..NORMAL TERMINATION ?
        BZ SEEK40     ..EXIT IF YES
        @IOCBPTR.AND.10H     ..GET DRIVE FAIL BIT
-  ***
        *2->@SP     ..LINE UP FOR STATUS
-  ***
        @IOCBPTR.AND.08H     ..GET DRIVE INACTIVE BIT
-  ***
        *2*2*2.OR.@     ..COMBINE WITH ABOVE
-  ***
        LSNZ;02H     ..IF NEITHER: GET BAD TERM BIT
-  ***
SEEK40    ->ASL.0;EXIT     ..LOAD RESULT STATUS: AND EXIT

..*************************************************************************
..
..      READ SECTOR ROUTINES (ALL MUST BE PRECEDED BY SEEK OR SEEKA)
..
..      READTR READS USING DMA ADDRESS FROM @ PARA AS DESCRIBED IN SEEK
..      HIGH NIBBLE OF UNIT # INDICATES # OF SECTORS TO READ (@512B/S).

READTR   CALL SETBC;DC RDCMD,DMAI
         EXIT

..       READST READS 1 SECTOR. USING DMA ADDRESS 0 PARA

READST   CALL SETRW;DC RDCMD,DMAI
         EXIT

..       READA READS 1 SECTOR, DOES NOT SETUP DMA POINTER

READA    CALL DORW; DC RDCMD, DMAI
         EXIT
..*************************************************************************

..       WRITE SECTOR ROUTINES (ALL MUST BE PRECEDED BY SEEK OR SEEKA)
..       ALL DO CRC READ AFTER WRITE IF WRITE TERMINATED OKAY

..       WRITTR WRITES MULTIPLE SECTORS AS DESCRIBED FOR READTR

WRITTR   CALL SETBC;DC WTCMD,DMAO
           BR CKFCRC

         ..WRITST WORKS LIKE READST

WRITST   CALL SETRW;DC WTCMD,DMAO
         BR CKFCRC

         ..WRITA WORKS LIKE READA

WRITA     CALL DORW;DC WTCMD,DMAO

         ..COMMON CRC CHECK

CKFCRC     ASL.0;BNZ WRITEX            ..IF TERMINATED OKAY
         CALL DORW
         DC RDCMD,CRCREAD            ..DO READ CRC
WRITEX     EXIT
..*************************************************************************

..             COMMON BYTE COUNT SETUP ENTER POINT

SETBC    A.0(IOCB+1)->IOCBPTR.0
         A.1(IOCB+1)->IOCBPTR.1      ..POINT AT BC IN IOCB
         @PARA.AND.0F0H              ..RETRIEVE 0 OF SECTORS AND STUFF
         /2/2->@IOCBPTR       ..BC (                                                                                                              BYTES = 128 X BC)
-  ***

..            COMMON SETUP DMA POINTER ENTER POINT

SETRW    INC PARA;INC PARA;INC PARA  ..POINT AT HI BUFFER ADDRESS BYTE
         @PARA!->DMAPTR.1           ..SET UP DMA OUTPUT BUFFER POINTER
-  ***
         @PARA->DMAPTR.0
         DEC PARA;DEC PARA           ..POINT BACK AT UNIT BYTE
         DEC PARA;DEC PARA

..               COMMON DO READ OR WRITE ROUTINE

..       CALL WITH IMMEDIATE BYTE - COMMAND. NEXT BYTE = DMA OPERATION
..	 CAN DO ANY 9 WORD COMMAND. LEAVES ASL.0 WITH RESULT STATUS
..	 MUST HAVE SEEK DONE FIRST FOR ENTIRE SETUP

DORW     A.0(IOCB+2)->IOCBPTR.0
         A.1(IOCB+2)->IOCBPTR.1       ..PT TO COMMAND IN IOCB
         @LINK!->@IOCBPTR        ..AND LOAD IT
-  ***
         DEC IOCBPTR;DEC IOCBPTR      ..POINT TO DMA DIRECTION
         @LINK!->@IOCBPTR       ..AND LOAD IT
-  ***
         9->CMDCNT.0       ..9 OUTPUTS FOR COMMAND
-  ***
         CALL CMD       ..GO DO IT
         CALL WAIT       ..AND GET RESULTS
         A.0(STA0)->IOCBPTR.0
         A.1(STA0)->IOCBPTR.1      ..POINT TO STATUS 0 BYTE
         @IOCBPTR.AND.0C0H       ..IF NORMAL TERMINATION
-  ***
         BZ DORWEX       ..GO EXIT
         @IOCBPTR.AND.10H       ..PUT DRIVE FAIL (STA0 BIT 4)
-  ***
         *2->@SP       ..INTO STATUS BIT 5
-  ***
         @IOCBPTR!.AND.08H       ..OR DR. INACTIVE (STAO BIT 3)
-  ***
         *2*2*2.OR.@->@SP       ..INTO STATUS BIT 6
-  ***
         @IOCBPTR.AND.02H       ..OR WRITE PROT. (STA1 BIT 1)
-  ***
         *2*2*2.OR.@->@SP       ..INTO STATUS BIT 4
-  ***
         @IOCBPTR!.AND.20H       ..CRC ERROR (STA1 BIT 5)
-  ***
         /2/2.OR.@->@SP      ..INTO STATUS BIT 3
         @IOCBPTR.AND.40H       ..OR FOUND DDM (STA2 BIT 6)
-  ***
         *2.OR.@       ..INTO STATUS BIT 7
-  ***
         LSNZ;02H       ..SET TERM. ERROR IF NONE ABOVE
-  ***
DORWEX     ->ASL.0;EXIT       ..STORE STATUS BYTE. EXIT


..*************************************************************************
..                  KEYBOARD READ / WRITE ROUTINE
..
..       USES SEEKA, WRITA, READA, RECAL, ENDSTRG, CKSTRG
..
WDISK    ASL.0;LSKP                   ..(ASL HAS ZEROED)
RDISK     1->@-       ..STORE READ./ WRITE FLAG
-  ***
         CALL SPECIFY       ..INITIALIZE 765
         CALL CKSTRG;DC 'A'       ..PROMPT FOR ADDRESS
         ASL.0->DMAPTR.0
         ASL.1->DMAPTR.1              ..STORE DESTINATION ADDRESS
         CALL CKSTRG;DC 'D'       ..PROMPT FOR DRIVE #
         ASL.0->@SP       ..STORE FOR RECAL AND SEEK
-  ***
         -4;BDF WREREX       ..ERROR IF DRIVE # TOO HIGH
         SP.0->PARA.0
         SP.1->PARA.1              ..POINT PARA @ DRIVE #
         DEC SP
         0->ASL.1                     ..SET FOR TRACK O
         CALL SEEKA       ..DO SEEK (DOES RECAL)
         ASL.0;BNZ DERROR       ..EXIT IF TERM ERROR
         CALL CKSTRG;DC 'T'       ..PROMPT FOR TRACK C
         ASL.0->@-       ..STORE FOR SEEK LATER
-  ***
         -MAXTRK;BDF WREREX       ..ERROR IF TRACK # TOO HIGH
         CALL ENDSTRG        ..PROMPT FOR SECTOR #
         DC 'S'       ..LAST PROMPT, END INPUTS HITH CR
         DEC ASL       ..SEEK WANTS SECTOR # -1
         ASL.0-MAXSEC;BDF WREREX      ..ERROR IF SECTOR # TOO HIGH
         INC SP;@SP->ASL.1       ..RETRIEVE TRACK #
-  ***
         SP.0->PARA.0
         SP.1->PARA.1
         INC PARA                     ..POINT PARA @ DRIVE # AGAIN
         CALL SEEKA       ..AND SEEK (SETS UP FOR READ/WRITE)
         ASL.0;BNZ DERROR       ..EXIT IF TERM ERROR
         INC SP;INC SP
         @SP;BZ CALWRT                ..IF READ FLAG SET.
         CALL READA       ..DO IT
         BR CKRDWR
CALWRT     CALL WRITA      ..OTHERWISE DO WRITE
CKRDWR     ASL.0;LBZ RENTER       ..SUCCESSFUL EXIT IF TERM OKAY
DERROR     CALL OSTRNG
         DC CR,LF,'DISK',0            ..OTHERWISE PRINT DISK ERROR

WREREX     LBR ERROR
..*************************************************************************
..                     KEYBOARD INPUT/OUTPUT ROUTINE
..
..       USES CKSTRG,ENDSTRG, USES AUX.0 FOR TEMP STORE
..
..	 OUTPUT ROUTINE
..
OUTPORT  CALL CKSTRG;DC 'G'           ..PROMPT FOR GROUP 3
         ASL.0->@-       ..STORE FOR LATER
-  ***
         CALL CKSTRG;DC 'P'       ..PROMPT FOR PORT #
         ASL.0.AND.07H;LBZ ERROR      ..GET PORT #5 ERROR IF 0
         .OR.60H->AUX.0          ..MAKE 6X INSTR., STORE
-  ***
         CALL ENDSTRG;DC 'B'       ..PROMPT FOR OUTPUT BYTE
         INC SP        ..POINT SP 0 GROUP 9
         OUT DISKSEL;DEC SP       ..OUTPUT IT. FIX STACK
         0D3H->@-        ..PUT RETURN (SEP R3) ON STACK
-  ***
         ASL.0->@-       ..PUT OUTPUT BYTE ON STACK
-  ***
         AUX.0->@SP       ..OUTPUT INSTRUCT. ON STACK
-  ***
         SEP SP       ..DO OUTPUT COMMAND
         LBR RENTER       ..BACK TO UTILITY (FIXES SP. GROUP 4)

..       INPUT ROUTINE
..
INPORT    CALL CKSTRG;DC 'G'           ..PROMPT FOR GROUP #
         ASL.0->@-       ..STORE FOR LATER
-  ***
         CALL ENDSTRG;DC 'P'       ..PROMPT FOR PORT #
         ASL.0.AND.07H;LBZ ERROR      ..ERROR IF INPUT PORT - o
         .OR.68H->@SP          ..MAKE INPUT INST., STORE
-  ***
         INC SP       ..POINT SP 2 GROUP 4
         OUT BDSEL;DEC SP       ..OUTPUT IT. FIX SP
         0D3H->@-            ..PUT RETURN (SEP R3) ON STACK
-  ***
         SEX PC       ..(THIS SO INPUT WON'T CLOBBER SP)
         SEP SP       ..DO INPUT
         DEC SP       ..FIX SP
         ->AUX.0       ..STORE INPUT FOR LATER
-  ***
         OUT BDSEL;DC TRMINL       ..SEL. TERMINAL
         CALL TYPE6;DC LF
         AUX.0->CHAR.1
         CALL TYPE2                   ..AND DISPLAY BYTE
         LBR RENTER

..*************************************************************************
..                    HEX STRING INPUT ROUTINE
..
..       TYPES PROMPT (FROM @ LK!) =
.. 	 ZEROES ASL;INPUTS CHARECTERS UNTIL NON-HEX INPUT
.. 	 CKSTRG GOES TO ERROR IF NOT SPACE
.. 	 ENDSTRG GOES TO ERROR IF NOT CR
..
CKSTRG     ' ';LSKP                     ..LOAD SPACE
ENDSTRG  CR->@-       ..OR CR FOR CHECK
-  ***
         @LINK!->CHAR.1
         CALL TYPE                    ..TYPE PROMPT
         CALL TYPE6;DC '=' 
         0->ASL.0,ASL.1               ..ZERO INPUT REGISTER
         CALL READHX       ..INPUT UNTIL 1ST NON HEX
         INC SP
         .XOR.@                       ..IF DOESN'T MATCH ABOVE
         LBNZ ERROR       ..ABORT
CKSTEX     EXIT
..
..*************************************************************************
..                       MONITOR SELF TEST ROUTINE
..       DOES CHECKSUM OF PROM; WRITES AND READS 55 AND AA TO ALL RAM
..
TEST     A.0(UT71)->IOCBPTR.0,@SP      ..ZERO CHECKSUM.
         A.1(UT71)->IOCBPTR.1        ..PT TO START OF PROM
-  ***
TROM1     @IOCBPTR!+@->@SP          ..ADD ALL PROM TOGETHER
-  ***
         IOCBPTR.1-A.1(RAMADR)
         BM TROM1                      ..LOOP UNTIL IT HITS RAM
         @SP;BZ RAMTEST               ..IF CHECK NOT = 0
         CALL OSTRNG
         DC LF,'ROM BAD',0             ..PRINT BAD ROM


         LBR RENTER           ..EXIT TO UTILITY
..
RAMTEST  A.1(UT71)->IOCBPTR.1
         DEC IOCBPTR                   ..POINT JUST BELOW PROM
         SEX IOCBPTR
RAM1     55H->@-
         IOCBPTR.1-A.1(RAMADR-1)
         BNZ RAM1                      ..FILL ALL RAM WITH 55
         INC IOCBPTR                   ..PT AT JUST ABOVE PROM
RAM2   @.XOR.0FFH->@IOCBPTR
         @!.XOR.0AAH
         BZ RAM2                       ..CHECK AA WROTE
         DEC IOCBPTR
         IOCBPTR.1-A.1(UT71)           ..IF FIRST FAILURE IS IN PROM
         BNZ BRAMM
         CALL OSTRNG                   ..PRINT OKAY
         DC LF,'MEMORY OK',0

         LBR RENTER                    ..AND EXIT
..
BRAMM   CALL OSTRNG                    ..BAD RAM MESSAGE
        DC LF,'RAM BAD, P',0



        IOCBPTR.1->CHAR.1
        CALL TYPE2                     ..PRINT PAGE #
        LBR RENTER        ..AND EXIT
..*************************************************************************
..      DISK I/O ENTRY TABLE AND ROM TEST CHECK BYTE

        ORG UT71+007D8H

CFRETS  LBR CFRET                 ..COMMAND FILE RETURN POINT
READTRS LBR READTR   ..READ MULTIPLE SECTOR, SET DMA FROM PARA.
WRITTRS LBR WRITTR   ..WRITE AS ABOVE
READAS  LBR READA   ..READ SECTOR USING ALREADY SET DMA POINTER
WRITAS  LBR WRITA   ..WRITE SECTOR AS ABOVE
RECALS  LBR RECAL   ..RECALIBRATE. USE DMAPTR.O FOR DR. I
SEEKAS  LBR SEEKA   ..SEEK BY TRACK & SECTOR IN ASL, @PARA = DR. #
CMDS    LBR CMD   ..OUTPUT COMMAND BYTES
RENTERS LBR RENTER   ..UT71 RENTRY ADDRESS
WAITS   LBR WAIT   ..SERVICE FDC AFTER COMMAND
SEEKS   LBR SEEKST   ..SEEK BY PSN IN PARA. BLOCK
READS   LBR READST   ..READ SECTOR. SET DMA FROM PARA. BLOCK
WRITS   LBR WRITST  ..WRITE SECTOR AS ABOVE
CHECK   DC 0FFH   ..CHECK SUM BYTE (SET AFTER ASSEMBLY)

        END

